SystemOrganization addCategory: #CompilerTests!

TestCase subclass: #ArrayLiteralTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ArrayLiteralTest methodsFor: 'running' stamp: 'EstebanLorenzano 8/3/2012 13:59'!
runCase

	SystemAnnouncer uniqueInstance suspendAllWhile: [ super runCase ]
! !

!ArrayLiteralTest methodsFor: 'initialization' stamp: 'avi 2/16/2004 21:09'!
tearDown
	self class removeSelector: #array! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'lr 1/29/2009 20:34'!
testByteArrayBase
	self class compile: 'array ^ #[2r1010 8r333 16rFF]'.
	self assert: (self array isKindOf: ByteArray).
	self assert: (self array size = 3).
	self assert: (self array first = 10).
	self assert: (self array second = 219).
	self assert: (self array last = 255)
	
! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'lr 1/29/2009 20:35'!
testByteArrayEmpty
	self class compile: 'array ^ #[]'.
	self assert: (self array isKindOf: ByteArray).
	self assert: (self array isEmpty)! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'lr 1/29/2009 20:36'!
testByteArrayLiteral
	self class compile: 'array ^ #[ 1 2 3 4 ]'.
	self assert: (self array = self array).
	self assert: (self array == self array)! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'lr 1/29/2009 20:45'!
testByteArrayLong
	self class compile: 'array ^ #[ ' , ((0 to: 255) inject: ' ' into: [ :r :e | r , ' ' , e asString ]) , ' ]'.
	self assert: (self array isKindOf: ByteArray).
	self assert: (self array size = 256).
	0 to: 255 do: [ :index | self assert: (self array at: index + 1) = index ]! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'lr 1/29/2009 20:32'!
testByteArrayRange
	self class compile: 'array ^ #[ 0 255 ]'.
	self assert: (self array isKindOf: ByteArray).
	self assert: (self array size = 2).
	self assert: (self array first = 0).
	self assert: (self array last = 255)! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'Henrik Sperre Johansen 3/23/2009 13:55'!
testByteArrayWithinArray
	self class compile: 'array ^ #( #[1] #[2] )'.
	self assert: (self array isKindOf: Array).
	self assert: (self array size = 2).
	self assert: (self array first isKindOf: ByteArray).
	self assert: (self array first first = 1).
	self assert: (self array last isKindOf: ByteArray).
	self assert: (self array last first = 2)
	
! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'avi 2/16/2004 21:08'!
testReservedIdentifiers
	self class compile: 'array ^ #(nil true false)'.
	self assert: self array = {nil. true. false}.! !

!ArrayLiteralTest methodsFor: 'tests' stamp: 'avi 2/16/2004 21:09'!
testSymbols
	self class compile: 'array ^ #(#nil #true #false #''nil'' #''true'' #''false'')'.
	self assert: self array = {#nil. #true. #false. #nil. #true. #false}.! !

TestCase subclass: #ClosureCompilerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/20/2008 09:40'!
methodWithCopiedAndAssignedTemps
	| blk "0w" a "0w" b "0w" c "0w" t "0w" r1 "0w" r2 "0w" |
	a := 1. "1w"
	b := 2. "1w"
	c := 4. "1w"
	t := 0. "1w"
	blk "5w" := ["2" t  "3w" := t "3r" + a "3r" + b "3r" + c "3r" ] "4".
	r1 "5w" := blk "5r" value.
	b "5w" := -100.
	r2 "5w" := blk "5r" value.
	^r1 "5r" -> r2 "5r" -> t "5r"

	"a: main(read(),write(0,1)), block(read(3),write()) => copy; no writes follow read
	 b: main(read(),write(0,1,5)), block(read(3),write()) => remote; write follows contained read
	 blk: main(read(5),write(0,5)), block(read(),write()) => no copy in blocks < 5
	 c: main(read(),write(0,1)), block(read(3),write()) => copy; no writes follow read
	 r1: main(read(5),write(0,5)), block(read(),write()) => no copy in blocks < 5
	 r2: main(read(5),write(0,5)), block(read(),write()) => no copy in blocks < 5
	 t: main(read(5),write(0,1)), block(read(3),write(3)) => remote; read follows contained write"


	"(Parser new
		encoderClass: EncoderForV3;
		parse: (self class sourceCodeAt: #methodWithCopiedAndAssignedTemps)
		class: self class) generateUsingClosures: #(0 0 0 0)"! !

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/19/2008 20:45'!
methodWithCopiedAndPostClosedOverAssignedTemps
	| blk a b c r1 r2 |
	a := 1.
	b := 2.
	c := 4.
	blk := [a + b + c].
	r1 := blk value.
	b := nil.
	r2 := blk value.
	r1 -> r2

	"(Parser new
		encoderClass: EncoderForV3;
		parse: (self class sourceCodeAt: #methodWithCopiedAndPostClosedOverAssignedTemps)
		class: self class) generateUsingClosures: #(0 0 0 0)"! !

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/19/2008 20:10'!
methodWithCopiedTemps
	| a b c r |
	a := 1.
	b := 2.
	c := 4.
	r := [a + b + c] value.
	b := nil.
	r

	"Parser new
		parse: (self class sourceCodeAt: #methodWithCopiedTemps)
		class: self class"

	"(Parser new
		encoderClass: EncoderForV3;
		parse: (self class sourceCodeAt: #methodWithCopiedTemps)
		class: self class) generateUsingClosures: #(0 0 0 0)"! !

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/19/2008 14:24'!
methodWithOptimizedBlocks
	| s c |
	s := self isNil
			ifTrue: [| a | a := 'isNil'. a]
			ifFalse: [| b | b := 'notNil'. b].
	c := String new: s size.
	1 to: s size do:
		[:i| c at: i put: (s at: i)].
	^c

	"Parser new
		parse: (self class sourceCodeAt: #methodWithOptimizedBlocks)
		class: self class"! !

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/19/2008 14:24'!
methodWithOptimizedBlocksA
	| s c |
	s := self isNil
			ifTrue: [| a | a := 'isNil'. a]
			ifFalse: [| a | a := 'notNil'. a].
	c := String new: s size.
	1 to: s size do:
		[:i| c at: i put: (s at: i)].
	^c

	"Parser new
		parse: (self class sourceCodeAt: #methodWithOptimizedBlocksA)
		class: self class"! !

!ClosureCompilerTest class methodsFor: 'code examples' stamp: 'eem 5/19/2008 14:12'!
methodWithVariousTemps
	| classes total totalLength |
	classes := self withAllSuperclasses.
	total := totalLength := 0.
	classes do: [:class| | className |
		className := class name.
		total := total + 1.
		totalLength := totalLength + className size].
	^total -> totalLength

	"Parser new
		parse: (self class sourceCodeAt: #methodWithVariousTemps)
		class: self class"! !

!ClosureCompilerTest methodsFor: 'source' stamp: 'MarcusDenke 4/29/2012 20:40'!
closureCases
	^#(
'| n |
n := 1.
^n + n'

'[:c :s| | mn |
mn := Compiler new
		compile: (c sourceCodeAt: s)
		in: c
		notifying: nil
		ifFail: [self halt].
mn generate: #(0 0 0 0).
{mn blockExtentsToTempsMap.
  mn encoder schematicTempNames}]
			value: AbstractInstructionTests
			value: #runBinaryConditionalJumps:'

'inject: thisValue into: binaryBlock
	| nextValue |
	nextValue := thisValue.
	self do: [:each | nextValue := binaryBlock value: nextValue value: each].
	^nextValue'

'mapFromBlockStartsIn: aMethod toTempVarsFrom: schematicTempNamesString constructor: aDecompilerConstructor
	| map |
	map := aMethod
				mapFromBlockKeys: aMethod startpcsToBlockExtents keys asSortedCollection
				toSchematicTemps: schematicTempNamesString.
	map keysAndValuesDo:
		[:startpc :tempNameTupleVector| | subMap tempVector numTemps |
		subMap := Dictionary new.
		tempNameTupleVector do:
			[:tuple|
			tuple last isArray
				ifTrue:
					[subMap at: tuple last first put: tuple last last.
					 numTemps := tuple last first]
				ifFalse:
					[numTemps := tuple last]].
		tempVector := Array new: numTemps.
		subMap keysAndValuesDo:
			[:index :size|
			tempVector at: index put: (Array new: size)].
		tempNameTupleVector do:
			[:tuple| | itv |
			tuple last isArray
				ifTrue:
					[itv := tempVector at: tuple last first.
					 itv at: tuple last last
						put: (aDecompilerConstructor
								codeTemp: tuple last last - 1
								named: tuple first)]
				ifFalse:
					[tempVector
						at: tuple last
						put: (aDecompilerConstructor
								codeTemp: tuple last - 1
								named: tuple first)]].
		subMap keysAndValuesDo:
			[:index :size|
			tempVector
				at: index
				put: (aDecompilerConstructor
						codeRemoteTemp: index
						remoteTemps: (tempVector at: index))].
		map at: startpc put: tempVector].
	^map'

 'gnuifyFrom: inFileStream to: outFileStream

	| inData beforeInterpret inInterpret inInterpretVars beforePrimitiveResponse inPrimitiveResponse |

	inData := inFileStream upToEnd withSqueakLineEndings.
	inFileStream close.

	outFileStream
		nextPutAll: ''/* This file has been post-processed for GNU C */'';
		cr; cr; cr.

	beforeInterpret := true.    "whether we are before the beginning of interpret()"
	inInterpret := false.     "whether we are in the middle of interpret"
	inInterpretVars := false.    "whether we are in the variables of interpret"
	beforePrimitiveResponse := true.  "whether we are before the beginning of primitiveResponse()"
	inPrimitiveResponse := false.   "whether we are inside of primitiveResponse"
	''Gnuifying''
		displayProgressAt: Sensor cursorPoint
		from: 1 to: (inData occurrencesOf: Character cr)
		during:
			[:bar | | lineNumber |
			lineNumber := 0.
			inData linesDo:
				[ :inLine | | outLine extraOutLine caseLabel |
				bar value: (lineNumber := lineNumber + 1).
				outLine := inLine. 	"print out one line for each input line; by default, print out the line that was input, but some rules modify it"
				extraOutLine := nil.   "occasionally print a second output line..."
				beforeInterpret ifTrue: [
					inLine = ''#include "sq.h"'' ifTrue: [
						outLine := ''#include "sqGnu.h"'' ].
					inLine = ''interpret(void) {'' ifTrue: [
						"reached the beginning of interpret"
						beforeInterpret := false.
						inInterpret := true.
						inInterpretVars := true ] ]
				ifFalse: [
				inInterpretVars ifTrue: [
					(inLine findString: ''register struct foo * foo = &fum;'') > 0 ifTrue: [
						outLine := ''register struct foo * foo FOO_REG = &fum;'' ].
					(inLine findString: '' localIP;'') > 0 ifTrue: [
						outLine := ''    char* localIP IP_REG;'' ].
					(inLine findString: '' localFP;'') > 0 ifTrue: [
						outLine := ''    char* localFP FP_REG;'' ].
					(inLine findString: '' localSP;'') > 0 ifTrue: [
						outLine := ''    char* localSP SP_REG;'' ].
					(inLine findString: '' currentBytecode;'') > 0 ifTrue: [
						outLine := ''    sqInt currentBytecode CB_REG;'' ].
					inLine isEmpty ifTrue: [
						"reached end of variables"
						inInterpretVars := false.
						outLine := ''    JUMP_TABLE;''.
						extraOutLine := inLine ] ]
				ifFalse: [
				inInterpret ifTrue: [
					"working inside interpret(); translate the switch statement"
					(inLine beginsWith: ''		case '') ifTrue: [
						caseLabel := (inLine findTokens: ''	 :'') second.
						outLine := ''		CASE('', caseLabel, '')'' ].
					inLine = ''			break;'' ifTrue: [
						outLine := ''			BREAK;'' ].
					inLine = ''}'' ifTrue: [
						"all finished with interpret()"
						inInterpret := false ] ]
				ifFalse: [
				beforePrimitiveResponse ifTrue: [
					(inLine beginsWith: ''primitiveResponse('') ifTrue: [
						"into primitiveResponse we go"
						beforePrimitiveResponse := false.
						inPrimitiveResponse := true.
						extraOutLine := ''    PRIM_TABLE;'' ] ]
				ifFalse: [
				inPrimitiveResponse ifTrue: [
					inLine = ''	switch (primitiveIndex) {'' ifTrue: [
						extraOutLine := outLine.
						outLine := ''	PRIM_DISPATCH;'' ].
					inLine = ''	switch (GIV(primitiveIndex)) {'' ifTrue: [
						extraOutLine := outLine.
						outLine := ''	PRIM_DISPATCH;'' ].
					(inLine beginsWith: ''	case '') ifTrue: [
						caseLabel := (inLine findTokens: ''	 :'') second.
						outLine := ''	CASE('', caseLabel, '')'' ].
					inLine = ''}'' ifTrue: [
						inPrimitiveResponse := false ] ]
				] ] ] ].

				outFileStream nextPutAll: outLine; cr.
				extraOutLine ifNotNil: [
					outFileStream nextPutAll: extraOutLine; cr ]]].

	outFileStream close' )! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 8/2/2011 17:30'!
doTestDebuggerTempAccessWith: one with: two
	"Test debugger access for temps"
	| outerContext local1 remote1 |
	outerContext := thisContext.
	local1 := 3.
	remote1 := 1/2.
	self assert: (self class evaluatorClass new evaluate: 'one' in: thisContext to: self) == one.
	self assert: (self class evaluatorClass new evaluate: 'two' in: thisContext to: self) == two.
	self assert: (self class evaluatorClass new evaluate: 'local1' in: thisContext to: self) == local1.
	self assert: (self class evaluatorClass new evaluate: 'remote1' in: thisContext to: self) == remote1.
	self class evaluatorClass new evaluate: 'local1 := -3.0' in: thisContext to: self.
	self assert: local1 = -3.0.
	(1 to: 2) do:
		[:i| | local2 r1 r2 r3 r4 |
		local2 := i * 3.
		remote1 := local2 / 7.
		self assert: thisContext ~~ outerContext.
		self assert: (r1 := self class evaluatorClass new evaluate: 'one' in: thisContext to: self) == one.
		self assert: (r2 := self new evaluate: 'two' in: thisContext to: self) == two.
		self assert: (r3 := self class evaluatorClass new evaluate: 'i' in: thisContext to: self) == i.
		self assert: (r4 := self class evaluatorClass new evaluate: 'local2' in: thisContext to: self) == local2.
		self assert: (r4 := self class evaluatorClass new evaluate: 'remote1' in: thisContext to: self) == remote1.
		self assert: (r4 := self class evaluatorClass new evaluate: 'remote1' in: outerContext to: self) == remote1.
		self class evaluatorClass new evaluate: 'local2 := 15' in: thisContext to: self.
		self assert: local2 = 15.
		self class evaluatorClass new evaluate: 'local1 := 25' in: thisContext to: self.
		self assert: local1 = 25.
		{ r1. r2. r3. r4 } "placate the compiler"].
	self assert: local1 = 25.
	self assert: remote1 = (6/7)! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 6/4/2008 16:58'!
supportTestSourceRangeAccessForDecompiledInjectInto: method source: source
	"Test debugger source range selection for inject:into:"
	^self
		supportTestSourceRangeAccessForInjectInto: method
		source: source
		selectionSequence: #(	':= t1'
								'do: [:t4 | t3 := t2 value: t3 value: t4]'
								'value: t3 value: t4'
								':= t2 value: t3 value: t4'
								']'
								'value: t3 value: t4'
								':= t2 value: t3 value: t4'
								']'
								'^t3')! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 6/4/2008 19:44'!
supportTestSourceRangeAccessForDecompiledNoBytecodeInjectInto: method source: source
	"Test debugger source range selection for inject:into:"
	^self
		supportTestSourceRangeAccessForInjectInto: method
		source: source
		selectionSequence: #(	'at: 1 put: t1'
								'do: [:t4 | t3 at: 1 put: (t2 value: (t3 at: 1) value: t4)]'
								'value: (t3 at: 1) value: t4'
								'at: 1 put: (t2 value: (t3 at: 1) value: t4)'
								']'
								'value: (t3 at: 1) value: t4'
								'at: 1 put: (t2 value: (t3 at: 1) value: t4)'
								']'
								'^t3 at: 1')! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'StephaneDucasse 3/15/2010 21:24'!
supportTestSourceRangeAccessForInjectInto: method source: source
	"Test debugger source range selection for inject:into:"
	^self
		supportTestSourceRangeAccessForInjectInto: method
		source: source
		selectionSequence: #(	':= thisValue'
								'do: [:each | nextValue := binaryBlock value: nextValue value: each]'
								'value: nextValue value: each'
								':= binaryBlock value: nextValue value: each'
								'nextValue := binaryBlock value: nextValue value: each'
								'value: nextValue value: each'
								':= binaryBlock value: nextValue value: each'
								'nextValue := binaryBlock value: nextValue value: each'
								'^nextValue')! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 4/29/2012 11:56'!
supportTestSourceRangeAccessForInjectInto: method source: source selectionSequence: selections
	"Test debugger source range selection for inject:into:"
	| evaluationCount sourceMap debugTokenSequence debugCount |
	DebuggerMethodMap voidMapCache.
	evaluationCount := 0.
	sourceMap := method debuggerMap abstractSourceMap.
	debugTokenSequence := selections collect: [:string| Scanner new scanTokens: string].
	debugCount := 0.
	thisContext
		runSimulated: [(1 to: 2)
						withArgs:
							{	0.
								[:sum :each|
								 evaluationCount := evaluationCount + 1.
								 sum + each]}
						executeMethod: method]
		contextAtEachStep:
			[:ctxt| | range debugTokens |
			(ctxt method == method
			and: ["Exclude the send of #closureCopy:copiedValues: and braceWith:with:
				    to create the block, and the #new: and #at:'s for the indirect temp vector.
				   This for compilation without closure bytecodes. (Note that at:put:'s correspond to stores)"
				(ctxt willSend
					and: [(#(closureCopy:copiedValues:  new: at: braceWith:with:) includes: ctxt selectorToSendOrSelf) not])
				"Exclude the store of the argument into the home context (for BlueBook blocks)
				 and the store of an indirection vector into an initial temp"
				or: [(ctxt willStore
					and: [(ctxt isBlock and: [ctxt pc = ctxt startpc]) not
					and: [(ctxt isBlock not and: [(ctxt abstractPC = 2)]) not]])
				or: [ctxt willReturn]]]) ifTrue:
				[debugTokens := debugTokenSequence at: (debugCount := debugCount + 1) ifAbsent: [#(bogusToken)].
				 self assert: (sourceMap includesKey: ctxt abstractPC).
				 range := sourceMap at: ctxt abstractPC ifAbsent: [(1 to: 0)].
				 self assert: (Scanner new scanTokens: (source copyFrom: range first to: range last)) = debugTokens]].
	self assert: evaluationCount = 2! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 4/29/2012 14:11'!
testBlockDoitDecompilation
	"Tests that decompile of a doit block with remote vars executes correcly"
	"Tests that decompilation of a Block, when 'method' of block is equivalent to that compiled by a DoIt, preserves the temp names "
	
	|blockSourceStream methodNode compiledMethod block decompiledBlock|
	blockSourceStream := '|x y| [:a :b | x := a. y := b. x + y]' readStream.
	methodNode := nil class evaluatorClass new 
						from: blockSourceStream class: nil class context: nil notifying: nil;
						translate: blockSourceStream noPattern: true ifFail: [nil].
	compiledMethod := methodNode generateWithSource.
	block := nil withArgs: #() executeMethod: compiledMethod.
	
	self shouldnt: [decompiledBlock := block decompile] raise: Error.
	self assert: '{[:a :b | 
x := a.
	y := b.
	x + y]}' equals: decompiledBlock printString
! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 9/27/2011 13:59'!
testBlockNumbering
	"Test that the compiler and CompiledMethod agree on the block numbering of a substantial doit."
	"self new testBlockNumbering"
	| methodNode method tempRefs |
	methodNode :=
		Parser new
			encoderClass: EncoderForV3PlusClosures;
			parse: 'foo
					| numCopiedValuesCounts classToRemove |
					numCopiedValuesCounts := Dictionary new.
					0 to: 32 do: [:i| numCopiedValuesCounts at: i put: 0].
					Transcript clear.
					Smalltalk allClasses remove: classToRemove; do:
						[:c|
						{c. c class} do:
							[:b|
							Transcript nextPut: b name first; endEntry.
							b selectorsAndMethodsDo:
								[:s :m| | pn |
								m isQuick not ifTrue:
									[pn := b parserClass new
												encoderClass: EncoderForV3PlusClosures;
												parse: (b sourceCodeAt: s)
												class: b.
									 pn generate.
									 [pn accept: nil]
										on: MessageNotUnderstood
										do: [:ex| | msg numCopied |
											msg := ex message.
											(msg selector == #visitBlockNode:
											 and: [(msg argument instVarNamed: ''optimized'') not]) ifTrue:
												[numCopied := (msg argument computeCopiedValues: pn) size.
												 numCopiedValuesCounts
													at: numCopied
													put: (numCopiedValuesCounts at: numCopied) + 1].
											msg setSelector: #==.
											ex resume: nil]]]]].
					numCopiedValuesCounts'
			class: Object.
	method := methodNode generate.
	tempRefs := methodNode encoder blockExtentsToTempsMap.
	self assert: tempRefs keys asSet = method startpcsToBlockExtents values asSet! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'nice 1/16/2010 17:12'!
testBlockNumberingForInjectInto
	"Test that the compiler and CompiledMethod agree on the block numbering of Collection>>inject:into:
	 and that temp names for inject:into: are recorded."
	"self new testBlockNumberingForInjectInto"
	| methodNode method tempRefs |
	methodNode := Parser new
						encoderClass: EncoderForV3PlusClosures;
						parse: (Collection sourceCodeAt: #inject:into:)
						class: Collection.
	method := methodNode generate.
	tempRefs := methodNode encoder blockExtentsToTempsMap.
	self assert: tempRefs keys asSet = method startpcsToBlockExtents values asSet.
	self assert: ((tempRefs includesKey: (0 to: 6))
				and: [((tempRefs at: (0 to: 6)) collect: [:e | e first]) hasEqualElements: {'thisValue'. 'binaryBlock'. 'nextValue'}]).
	self assert: ((tempRefs includesKey: (2 to: 4))
				and: [((tempRefs at: (2 to: 4)) collect: [:e | e first])  hasEqualElements: {'each'. 'binaryBlock'.  'nextValue'}])! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:41'!
testDebuggerTempAccess
	<expectedFailure>
	self doTestDebuggerTempAccessWith: 1 with: 2! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenke 4/29/2012 20:41'!
testDecompiledDoitMethodTempNames
	"self new testDecompiledDoitMethodTempNames"
	"Test that a decompiled doit that has been copied with temps decompiles to the input"
	| removeComments |
	removeComments := [:n| n comment: nil].
	self closureCases do:
		[:source| | mns m mps mnps |
		"Need to compare an ungenerated tree with the generated method's methodNode
		 because generating code alters the tree when it introduces remote temp vectors."
		mns := #(first last) collect:
					[:ignored|
					source first isLetter
						ifTrue:
							[self class compilerClass new
								compile: source
								in: self class
								notifying: nil
								ifFail: [self error: 'compilation error']]
						ifFalse:
							[self class compilerClass new
								compileNoPattern: source
								in: self class
								context: nil
								notifying: nil
								ifFail: [self error: 'compilation error']]].
		m := (mns last generateWithSource).
		removeComments value: mns first.
		mns first nodesDo: removeComments.
		self assert: (mnps := mns first printString) = (mps := (m methodNode nodesDo: removeComments) printString)]! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:41'!
testInjectIntoDecompilations
	"Test various compilations decompile to the same code for a method sufficiently
	 simple that this is possible and sufficiently complex that the code generated
	 varies between the compilations."
	"self new testInjectIntoDecompilations"
	
	<expectedFailure>
	
	| source |
	source := (Collection sourceCodeAt: #inject:into:) asString.
	{ Encoder.
	   EncoderForV3. EncoderForLongFormV3.
	   EncoderForV3PlusClosures. EncoderForLongFormV3PlusClosures } do:
		[:encoderClass| | method |
		method := (Parser new
							encoderClass: encoderClass;
							parse: source
							class: Collection)
						generate.
		self assert: (Scanner new scanTokens: method decompileString)
					= #(inject: t1 into: t2
							| t3 |
							t3 ':=' t1 .
							self do: [ ':t4' | t3 ':=' t2 value: t3 value: t4 ] .
							^ t3)]! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:41'!
testInjectIntoDecompiledDebugs
	"Test various debugs of the decompiled form debug correctly."
	"self new testInjectIntoDecompiledDebugs"
	| source |
	
	<expectedFailure>
	
	source := (Collection sourceCodeAt: #inject:into:) asString.
	{ Encoder.
	   EncoderForV3PlusClosures. EncoderForLongFormV3PlusClosures } do:
		[:encoderClass| | method |
		method := (Parser new
							encoderClass: encoderClass;
							parse: source
							class: Collection)
						generate.
		self supportTestSourceRangeAccessForDecompiledInjectInto: method source: method decompileString]! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 7/24/2009 11:51'!
testInlineBlockCollectionEM1
	| a1 b1 i1 a2 b2 i2 we wb |
	b1 := OrderedCollection new.
	i1 := 1.
	[a1 := i1.
	 i1 <= 3] whileTrue:
		[b1 add: [a1].
		i1 := i1 + 1].
	b1 := b1 asArray collect: [:b | b value].
	b2 := OrderedCollection new.
	i2 := 1.
	we := [a2 := i2. i2 <= 3].
	wb := [b2 add: [a2]. i2 := i2 + 1].
	we whileTrue: wb. "defeat optimization"
	b2 := b2 asArray collect: [:b | b value].
	self assert: b1 = b2! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 3/7/2009 11:25'!
testInlineBlockCollectionLR1
	"Test case from Lukas Renggli"
	| col |
	col := OrderedCollection new.
	1 to: 11 do: [ :each | col add: [ each ] ].
	self assert: (col collect: [ :each | each value ]) asArray = (1 to: 11) asArray! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 3/7/2009 11:39'!
testInlineBlockCollectionLR2
	"Test case from Lukas Renggli"
	| col |
	col := OrderedCollection new.
	1 to: 11 do: [ :each | #(1) do: [:ignored| col add: [ each ]] ].
	self assert: (col collect: [ :each | each value ]) asArray = (1 to: 11) asArray! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 3/9/2009 11:00'!
testInlineBlockCollectionLR3
	| col |
	col := OrderedCollection new.
	1 to: 11 do: [ :each | | i | i := each. col add: [ i ]. i := i + 1 ].
	self assert: (col collect: [ :each | each value ]) asArray = (2 to: 12) asArray! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 7/22/2009 16:55'!
testInlineBlockCollectionSD1
	| a1 b1 a2 b2 |
	b1 := OrderedCollection new.
	1 to: 3 do:
		[:i |
		a1 := i.
		b1 add: [a1]].
	b1 := b1 asArray collect: [:b | b value].
	b2 := OrderedCollection new.
	1 to: 3 do:
		[:i |
		a2 := i.
		b2 add: [a2]] yourself. "defeat optimization"
	b2 := b2 asArray collect: [:b | b value].
	self assert: b1 = b2! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 9/5/2009 17:33'!
testOptimizedBlockLocalNilling1
	"Whether a block is optimized or not a block-local temp
	 should be nil at the start of each evaluation of the block."

	1 to: 3 do: [:i| | j |
		self assert: j isNil.
		j := i + 1.
		self assert: j isNil not]! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 9/5/2009 17:32'!
testOptimizedBlockLocalNilling2
	"Whether a block is optimized or not a block-local temp
	 should be nil at the start of each evaluation of the block."

	1 to: 6 do: [:i| | j k |
		self assert: j isNil.
		self assert: k isNil.
		i even
			ifTrue: [j := i + 2]
			ifFalse: [k := i + 1].
		self assert: (j isNil or: [k isNil]).
		self assert: (j isNil not or: [k isNil not])]! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 6/4/2008 15:20'!
testSourceRangeAccessForClosureBytecodeInjectInto
	"Test debugger source range selection for inject:into: for a version compiled with closures"
	"self new testSourceRangeAccessForClosureBytecodeInjectInto"
	| source method |
	source := (Collection sourceCodeAt: #inject:into:) asString.
	method := (Parser new
						encoderClass: EncoderForV3PlusClosures;
						parse: source
						class: Collection)
					generate: (Collection compiledMethodAt: #inject:into:) trailer.
	self supportTestSourceRangeAccessForInjectInto: method source: source! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 6/4/2008 15:20'!
testSourceRangeAccessForClosureLongFormBytecodeInjectInto
	"Test debugger source range selection for inject:into: for a version compiled with closures"
	"self new testSourceRangeAccessForClosureLongFormBytecodeInjectInto"
	| source method |
	source := (Collection sourceCodeAt: #inject:into:) asString.
	method := (Parser new
						encoderClass: EncoderForLongFormV3PlusClosures;
						parse: source
						class: Collection)
					generate: (Collection compiledMethodAt: #inject:into:) trailer.
	self supportTestSourceRangeAccessForInjectInto: method source: source! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'eem 6/4/2008 11:40'!
testSourceRangeAccessForInjectInto
	"Test debugger source range selection for inject:into: for the current version of the method"
	"self new testSourceRangeAccessForInjectInto"
	self supportTestSourceRangeAccessForInjectInto: (Collection compiledMethodAt: #inject:into:)
		source: (Collection sourceCodeAt: #inject:into:) asString! !

!ClosureCompilerTest methodsFor: 'tests' stamp: 'nice 1/16/2010 17:01'!
testTempNameAccessForInjectInto
	"self new testTempNameAccessForInjectInto"
	| methodNode method evaluationCount block debuggerMap |
	methodNode := Parser new
						encoderClass: EncoderForV3PlusClosures;
						parse: (Collection sourceCodeAt: #inject:into:)
						class: Collection.
	method := methodNode generate.
	debuggerMap := DebuggerMethodMap forMethod: method methodNode: methodNode.
	evaluationCount := 0.
	block := [:prev :each| | theContext tempNames |
			evaluationCount := evaluationCount + 1.
			theContext := thisContext sender.
			tempNames := debuggerMap tempNamesForContext: theContext.
			self assert: (tempNames hasEqualElements: tempNames).
			#('thisValue' 'each' 'binaryBlock' 'nextValue')
				with: { 0. each. block. prev}
				do: [:tempName :value|
					self assert: (debuggerMap namedTempAt: (tempNames indexOf: tempName) in: theContext) == value.
					tempName ~= 'each' ifTrue:
						[self assert: (debuggerMap namedTempAt: (tempNames indexOf: tempName) in: theContext home) == value]]].
	(1 to: 10) withArgs: { 0. block } executeMethod: method.
	self assert: evaluationCount = 10! !

ClosureCompilerTest subclass: #MethodHighlightingTests
	instanceVariableNames: 'creator timeStamp duration tracks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!MethodHighlightingTests methodsFor: 'tests' stamp: 'nice 1/5/2010 15:59'!
asXML
	"This method is just used as an example for #testMethodHighlighting."
	
	
	^String streamContents:[:s| | writer |
		writer := nil.
		writer xmlDeclaration: '1.0'.
		writer startTag: 'recording'; endTag.
			writer tag: 'creator' pcData: creator.
			writer tag: 'timestamp' pcData: timeStamp.
			writer tag: 'duration' pcData: duration.
			writer startTag: 'tracks'; endTag.
				tracks do:[:tdata|
					writer startTag: 'track'; attribute: 'type' value: tdata value; endTag.
					writer pcData: tdata key.
					writer endTag: 'track'.
				].
			writer endTag: 'tracks'.
		writer endTag: 'recording'.
	].
! !

!MethodHighlightingTests methodsFor: 'tests' stamp: 'AdrianLienhard 10/17/2009 16:43'!
testMethodHighlighting
	| map before after method retpc |
	"Test the highlighting of the asXML method.  Test the highlighting of the return
	 statement which should include the whole block supplied to streamContents:."
	"DebuggerMethodMap voidMapCache"
	"DebuggerMethodMap forMethod: MethodHighlightingTests >> #asXML"
	method := MethodHighlightingTests >> #asXML.
	map := DebuggerMethodMap forMethod: method.
	retpc := method endPC.
	before := map rangeForPC: retpc contextIsActiveContext: false.
	map instVarNamed: 'abstractSourceRanges' put: nil.
	after := map rangeForPC: retpc contextIsActiveContext: false.
	self assert: before size > 500.
	self assert: before = after! !

TestCase subclass: #ClosureTests
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ClosureTests methodsFor: 'utilities' stamp: 'StefanMarr 3/16/2012 01:36'!
assertValues: anArray
	| values |
	values := collection collect: [ :each | each value ].
	self 
		assert: anArray asArray = values asArray
		description: ['Expected: ' , anArray asArray printString , 
			', but got ' , values asArray printString]! !

!ClosureTests methodsFor: 'testing' stamp: 'lr 3/11/2009 17:35'!
methodArgument: anObject
	^ [ anObject ]
	! !

!ClosureTests methodsFor: 'running' stamp: 'lr 3/9/2009 16:48'!
setUp
	super setUp.
	collection := OrderedCollection new! !

!ClosureTests methodsFor: 'testing' stamp: 'lr 3/11/2009 17:33'!
testBlockArgument
	| block block1 block2 |
	block := [ :arg | | temp | temp := arg. [ temp ] ].
	block1 := block value: 1.
	block2 := block value: 2.
	self assert: block1 value = 1.
	self assert: block2 value = 2! !

!ClosureTests methodsFor: 'testing' stamp: 'lr 3/11/2009 17:33'!
testBlockTemp
	| block block1 block2 |
	block := [ :arg | [ arg ] ].
	block1 := block value: 1.
	block2 := block value: 2.
	self assert: block1 value = 1.
	self assert: block2 value = 2! !

!ClosureTests methodsFor: 'testing-empty' stamp: 'jb 7/1/2011 10:38'!
testEmptyBlockOneArgument
	self
		assert: (self class evaluatorClass evaluate: '[ :a ] value: 1') isNil
		description: 'Empty blocks in ST-80 should return nil'.
	self
		assert: (self class evaluatorClass evaluate: '[ :a | ] value: 1') isNil
		description: 'Empty blocks in ST-80 should return nil'.
	self
		assert: (self class evaluatorClass evaluate: '[ :a | | t | ] value: 1') isNil
		description: 'Empty blocks in ST-80 should return nil'! !

!ClosureTests methodsFor: 'testing-empty' stamp: 'jb 7/1/2011 10:42'!
testEmptyBlockTwoArguments
	self
		assert: (self class evaluatorClass evaluate: '[ :a :b ] value: 1 value: 2') isNil
		description: 'Empty blocks in ST-80 should return nil'.
	self
		assert: (self class evaluatorClass evaluate: '[ :a :b | ] value: 1 value: 2') isNil
		description: 'Empty blocks in ST-80 should return nil'.
	self
		assert: (self class evaluatorClass evaluate: '[ :a :b | | t | ] value: 1 value: 2') isNil
		description: 'Empty blocks in ST-80 should return nil'! !

!ClosureTests methodsFor: 'testing-empty' stamp: 'jb 7/1/2011 10:42'!
testEmptyBlockZeroArguments
	self
		assert: (self class evaluatorClass evaluate: '[ ] value') isNil
		description: 'Empty blocks in ST-80 should return nil'.
	self
		assert: (self class evaluatorClass evaluate: '[ | t | ] value') isNil
		description: 'Empty blocks in ST-80 should return nil'! !

!ClosureTests methodsFor: 'testing' stamp: 'sd 6/8/2012 23:31'!
testIsClean
	 | local |
	 local := #testIsClean.
	self assert: [] isClean. "closes over nothing at all"
	self assert: [:a :b| a < b] isClean. "accesses only arguments"
	self assert: [:a :b| | s | s := a + b. s even] isClean. "accesses only local variables"
	self deny: [^nil] isClean. "closes over home (^-return)"
	self deny: [self] isClean. "closes over the receiver"
	self deny: [collection] isClean. "closes over the receiver (to access the inst var collection)"
	self deny: [local] isClean. "closes over local variable of outer context"! !

!ClosureTests methodsFor: 'testing' stamp: 'lr 3/11/2009 17:36'!
testMethodArgument
	| temp block |
	temp := 0.
	block := [ [ temp ] ].
	temp := 1.
	block := block value.
	temp := 2.
	self assert: block value = 2! !

!ClosureTests methodsFor: 'testing' stamp: 'lr 3/11/2009 17:36'!
testMethodTemp
	| block1 block2 |
	block1 := self methodArgument: 1.
	block2 := self methodArgument: 2.
	self assert: block1 value = 1.
	self assert: block2 value = 2! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoArgument
	1 to: 5 do: [ :index |
		collection add: [ index ] ].
	self assertValues: #(1 2 3 4 5)! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoArgumentNotInlined
	| block |
	block := [ :index |
		collection add: [ index ] ].
	1 to: 5 do: block.
	self assertValues: #(1 2 3 4 5)! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoInsideTemp
	1 to: 5 do: [ :index | 
		| temp | 
		temp := index. 
		collection add: [ temp ] ].
	self assertValues: #(1 2 3 4 5)! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoInsideTempNotInlined
	| block |
	block := [ :index | 
		| temp | 
		temp := index. 
		collection add: [ temp ] ].
	1 to: 5 do: block.
	self assertValues: #(1 2 3 4 5)! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoOutsideTemp
	| temp |
	1 to: 5 do: [ :index | 
		temp := index. 
		collection add: [ temp ] ].
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-todo' stamp: 'lr 3/10/2009 14:51'!
testToDoOutsideTempNotInlined
	| block temp |
	block := [ :index | 
		temp := index. 
		collection add: [ temp ] ].
	1 to: 5 do: block.
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:29'!
testWhileModificationAfter
	| index |
	index := 0.
	[ index < 5 ] whileTrue: [
		collection add: [ index ].
		index := index + 1 ].
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:29'!
testWhileModificationAfterNotInlined
	| index block |
	index := 0.
	block := [ 
		collection add: [ index ].
		index := index + 1 ].
	[ index < 5 ] whileTrue: block.
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:29'!
testWhileModificationBefore
	| index |
	index := 0.
	[ index < 5 ] whileTrue: [ 
		index := index + 1.
		collection add: [ index ] ].
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:29'!
testWhileModificationBeforeNotInlined
	| index block |
	index := 0.
	block := [ 
		index := index + 1.
		collection add: [ index ] ].
	[ index < 5 ] whileTrue: block.
	self assertValues: #(5 5 5 5 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:52'!
testWhileWithTemp
	| index |
	index := 0.
	[ index < 5 ] whileTrue: [
		| temp |
		temp := index := index + 1.
		collection add: [ temp ] ].
	self assertValues: #(1 2 3 4 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'StephaneDucasse 7/3/2010 22:21'!
testWhileWithTempIsNil
	"self debug: #testWhileWithTempIsNil"
	| index |
	index := 0.
	[ index < 5 ] whileTrue: [
		| temp |
		collection add: temp.
		temp := index := index + 1. 
		collection add: temp].
	self assertValues: #(nil 1 nil 2 nil 3 nil 4 nil 5)! !

!ClosureTests methodsFor: 'testing-while' stamp: 'lr 3/10/2009 14:53'!
testWhileWithTempNotInlined
	| index block |
	index := 0.
	block := [
		| temp |
		temp := index := index + 1.
		collection add: [ temp ] ].
	[ index < 5 ] whileTrue: block.
	self assertValues: #(1 2 3 4 5)! !

TestCase subclass: #CompilerEvaluationTest
	instanceVariableNames: 'weightedAverage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CompilerEvaluationTest commentStamp: 'StephaneDucasse 6/9/2010 20:54' prior: 0!
This test is important since it covers a bug of during the debugging of super
binding. 
It uses the class AverageCost/TotalCost and WeightedAverageCost.!

!CompilerEvaluationTest methodsFor: 'helper methods' stamp: 'carlaGriggio 5/22/2010 21:24'!
method
	^self methodContext method asString
! !

!CompilerEvaluationTest methodsFor: 'helper methods' stamp: 'StephaneDucasse 6/9/2010 20:55'!
methodContext
	| process |
	process := [ self weightedAverage total ] newProcess.
	[ process step ] doWhileTrue: [ process suspendedContext method selector ~= #total ].
	^ process suspendedContext! !

!CompilerEvaluationTest methodsFor: 'helper methods' stamp: 'IgorStasenko 4/15/2011 17:12'!
model
	^ Smalltalk tools debugger context: self methodContext ! !

!CompilerEvaluationTest methodsFor: 'setup' stamp: 'StephaneDucasse 6/9/2010 20:55'!
setUp
	weightedAverage := WeightedAverageCost new ! !

!CompilerEvaluationTest methodsFor: 'testing' stamp: 'jb 7/1/2011 10:45'!
testEvaluationOfSelfSend
	"Tests the evaluation of an inherited method that uses super send
	and self send"
	"self debug: #testEvaluationOfSelfSend"
	| result |
	result := Compiler new
				evaluate: (self method copyFrom: 6 to: self method size)
				in: self model doItContext
				to: self model doItReceiver
				notifying: nil 
				ifFail: [^ #failedDoit]
				logged: true.
	self assert: result = (6250 / 3)! !

!CompilerEvaluationTest methodsFor: 'helper methods' stamp: 'carlaGriggio 5/23/2010 20:18'!
weightedAverage 
	^weightedAverage ! !

TestCase subclass: #CompilerExceptionsTest
	instanceVariableNames: 'status text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CompilerExceptionsTest methodsFor: 'compiling' stamp: 'StephaneDucasse 2/10/2011 16:59'!
compile: sourceString

       text := sourceString.
       self class
               compileSilently: text
               classified: 'generated'
               notifying: self! !

!CompilerExceptionsTest methodsFor: 'compiling' stamp: 'StephaneDucasse 2/10/2011 17:00'!
compiling: sourceCode shouldRaise: exceptionClass

	self should: [ self compile: sourceCode ] raise: exceptionClass! !

!CompilerExceptionsTest methodsFor: 'compiling' stamp: 'JohanBrichau 4/15/2011 15:56'!
interactive 
	^ true! !

!CompilerExceptionsTest methodsFor: 'compiling' stamp: 'StephaneDucasse 2/10/2011 17:01'!
removeGeneratedMethods

	self class removeCategory: 'generated'! !

!CompilerExceptionsTest methodsFor: 'emulating' stamp: 'cwp 8/25/2009 20:23'!
select
	! !

!CompilerExceptionsTest methodsFor: 'emulating' stamp: 'cwp 8/25/2009 20:23'!
selectFrom: start to: end 
	! !

!CompilerExceptionsTest methodsFor: 'emulating' stamp: 'cwp 8/25/2009 20:22'!
selectionInterval
	^ 1 to: 0! !

!CompilerExceptionsTest methodsFor: 'setUp' stamp: 'StephaneDucasse 2/10/2011 17:35'!
setUp

	self removeGeneratedMethods.
	status := Parser warningAllowed.
	Parser warnUser.! !

!CompilerExceptionsTest methodsFor: 'setUp' stamp: 'StephaneDucasse 3/15/2010 21:11'!
tearDown
	
	Parser warningAllowed: status
	! !

!CompilerExceptionsTest methodsFor: 'tests' stamp: 'JohanBrichau 4/15/2011 15:58'!
testAmbiguousSelector
       self
               compiling: 'griffle ^1--1'
               shouldRaise: AmbiguousSelector;
               compiling: 'griffle ^1@-1'
               shouldRaise: AmbiguousSelector;
               compiling: 'griffle ^1+-1'
               shouldRaise: AmbiguousSelector! !

!CompilerExceptionsTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/10/2011 17:29'!
testUndeclaredVariable

	self
		compiling: 'griffle ^ goo' shouldRaise: UndeclaredVariable;
		compiling: 'griffle ^ [ goo ] value' shouldRaise: UndeclaredVariable! !

!CompilerExceptionsTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/10/2011 17:30'!
testUndefinedVariable

	self
		compiling: 'griffle | goo | ^ goo' shouldRaise: UndefinedVariable;
		compiling: 'griffle [ | goo | ^ goo ] value' shouldRaise: UndefinedVariable! !

!CompilerExceptionsTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/10/2011 17:30'!
testUnknownSelector

	self
		compiling: 'griffle self reallyHopeThisIsntImplementedAnywhere' shouldRaise: UnknownSelector;
		compiling: 'griffle [ self reallyHopeThisIsntImplementedAnywhere ] value' shouldRaise: UnknownSelector! !

!CompilerExceptionsTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/10/2011 17:37'!
testUnusedVariable

       self
               compiling: 'griffle | goo | ^nil' shouldRaise: UnusedVariable.

	" does not work for the moment.
      self       compiling: 'griffle ^[ | goo | ]' shouldRaise: UnusedVariable
	"! !

!CompilerExceptionsTest methodsFor: 'emulating' stamp: 'StephaneDucasse 2/10/2011 17:36'!
text
	^ text! !

TestCase subclass: #CompilerNotifyingTest
	instanceVariableNames: 'text morph expectedErrors expectedErrorPositions failure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CompilerNotifyingTest commentStamp: 'nice 2/23/2012 22:09' prior: 0!
A CompilerNotifyingTest is a TestCase for checking that Compiler/Parser notifications are inserted at the right place in a TextEditor.

Instance Variables
	expectedErrorPositions:		<Array of: Integer>
	expectedErrors:		<Array of: String>
	failure:		<Object>
	morph:		<TextMorph>
	text:		<String>

errorPositions
	- the position where error text should be inserted for each chunk of text evaluated

errors
	- the error text that should be inserted on evaluation of each chunk of text evaluated

failure
	- an object returned in case of evaluation error and whose identity can be uniquely recognized as a failure
	
morph
	- the Morph holding the text
	
text
	- the string containing all the chunks to be evaluated (separated by %)
	  and the expected error messages (`enclosed in back quotes`)
	  this text will be stripped of the error messages before being evaluated.

!

!CompilerNotifyingTest methodsFor: 'private' stamp: 'nice 2/22/2012 00:56'!
enumerateAllSelections
	1 to: self numberOfSelections do: [:n |
		self assert: (self evaluateSelectionNumber: n) == failure.
		self assert: ((expectedErrors at: n) = morph editor selection asString).
		self assert: ((expectedErrorPositions at: n) = morph editor startIndex).
		morph editor cut].! !

!CompilerNotifyingTest methodsFor: 'private' stamp: 'nice 2/21/2012 23:52'!
evaluateSelection
	^(nil class evaluatorClass new)
		evaluate: morph editor selectionAsStream
		in: nil
		to: nil
		notifying: morph editor
		ifFail: [^failure]
		logged: false
	! !

!CompilerNotifyingTest methodsFor: 'private' stamp: 'nice 2/21/2012 23:58'!
evaluateSelectionNumber: n
	| i start stop |
	i := start := 1.
	[stop := morph text indexOf: $% startingAt: start + 1 ifAbsent: morph text size + 1.
	i = n]
		whileFalse:
			[i := i + 1.
			start := stop + 1].
	morph editor selectFrom: start to: stop - 1.
	^self evaluateSelection
	! !

!CompilerNotifyingTest methodsFor: 'accessing' stamp: 'nice 2/23/2012 22:32'!
expectedFailures
	"For these tests, the Compiler does not insert the notification where it would be most informative."
	
	^#(
		"Notifying before the literal (which can be a message send) would be more informative"
		testTooManyLiterals
		
		"Notifying right before the 16th argument would be informative"
		testTooManyArguments
		
		"Notifying before the extraneous argument in the right block woul be useful"
		testifTrueBlockWithArgument  testCaseOtherwiseBlockWithArgument
		
		"Pharo still accept the non standard unichar symbol syntax #) while every other Smalltalk would require quotes #')'"
		testInvalidLiteralCharacter
		)! !

!CompilerNotifyingTest methodsFor: 'initialize-release' stamp: 'nice 2/21/2012 23:52'!
initializeTextWithoutError
	"Remove the errors from the text to be compiled and answer the text without errors.
	Meanwhile, collect the expected error messages and their expected position."

	| input output errorStream positionStream |
	input := text readStream.
	output := (String new: text size) writeStream.
	errorStream := (Array new: self numberOfSelections) writeStream.
	positionStream := (Array new: self numberOfSelections) writeStream.
	
	[output nextPutAll: (input upTo: $`).
	input atEnd]
		whileFalse:
			[positionStream nextPut: output position + 1.
			errorStream nextPut: (input upTo: $`)].
	expectedErrors := errorStream contents.
	expectedErrorPositions := positionStream contents.
	^output contents! !

!CompilerNotifyingTest methodsFor: 'private' stamp: 'nice 2/21/2012 22:23'!
numberOfSelections
	^(text occurrencesOf: $%) + 1! !

!CompilerNotifyingTest methodsFor: 'initialize-release' stamp: 'nice 2/22/2012 00:54'!
setUp
	failure := Object new.! !

!CompilerNotifyingTest methodsFor: 'initialize-release' stamp: 'nice 2/21/2012 23:41'!
setUpForErrorsIn: aTextWithErrorsEnclosedInBackQuote
	"Extract the expectedErrors, the expectedErrorPositions and set up a TextMorph containing the text without errors.
	each section separated by % in aTextWithErrorsEnclosedInBackQuote will be evaluated separately.
	The expected error message should lie in aTextWithErrorsEnclosedInBackQuote at the expected position, and enclosed in back quotes."
	text := aTextWithErrorsEnclosedInBackQuote.
	morph := TextMorph new contents: self initializeTextWithoutError asText.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:44'!
testATempShadowingAnotherTemp
	self setUpForErrorsIn: '| x | x := 1. ^[ | ` Name is already defined ->`x | x ]'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 21:32'!
testAlltogether
	"Initialize for all known error conditions.
	Note that the chunk to be evaluated are separated by %, and expected errors enclosed in back quotes."
	
	self setUpForErrorsIn: '  "First, the senders of #offEnd:"
#` Unmatched bracket ->`[ 1 2 %
#[ 1 2 ` 8-bit integer or right bracket expected ->`256 4]%
$` A Character was expected ->`%
1+2   ` Unmatched comment quote ->`"unfinished comment%
#` Unmatched parenthesis ->`( 1 2%
#` Unmatched parenthesis ->`( 1 2 %
^nil printString ,  ` Unmatched string quote ->`''unfinished string%
	"Then, the senders of #expected:"
2r` a digit between 0 and 1 expected ->`3%
| x | x := ` Expression expected ->`%
[ :x : ` Argument name expected ->`1]%
[ :x ` Vertical bar expected ->`x + 1 ]%
[:x | 1 ` Period or right bracket expected ->`( 1 ) ]%
{ 1. 2` Period or right brace expected ->`%
{ 1. 2 ` Period or right brace expected ->`%
{ 1. 2 ` Period or right brace expected ->`3 %
{ 1. 2. ` Variable or expression expected ->`| x | %
super yourself` Cascading not expected ->`; yourself%
nil yourself; ` Cascade expected ->`^ 2%
	"#externalFunctionDeclaration is skipped, this cannot be evaluated"
1 to: ` Argument expected ->`:=%
1 +` Argument expected ->`%
1 + ` Argument expected ->`* 2 + 3%
1+(2 ` right parenthesis expected ->`.  %
1 + 2 ` Nothing more expected ->`^nil%
	"#pattern:inContext: skipped, cannot be evaluated"
	"#pragmaLiteral: #pragmaSequence #pragmaStatement skipped, cannot be evaluated"
(  ` expression expected ->`. 2  . )%
( 1  ` right parenthesis expected ->`. 2  . )%
	"#primitive:error: #primitive:module:error: skipped, cannot be evaluated"
^ ` Expression to return expected ->`. 1 + 2%
[ ^1 ` End of block expected ->`2]%
| x y ` Vertical bar expected ->`%
[:z | | x y ` Vertical bar expected ->`]%
1` an integer greater than 1 as valid radix expected ->`r0'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:06'!
testAssignmentOfSelf
	self setUpForErrorsIn: '` Cannot store into ->`self := 1. ^self'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:54'!
testCascadeInASuperSend
	self setUpForErrorsIn: 'super yourself` Cascading not expected ->`; yourself'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing-block arguments' stamp: 'nice 2/22/2012 01:15'!
testCaseOtherwiseBlockWithArgument
	self setUpForErrorsIn: 'nil caseOf: { [nil] -> [1] } otherwise: [:x` <- otherwise arg of caseOf:otherwise: has too many arguments ->` | 2 ]. ^nil '.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testDigitTooLargeForARadix
	self setUpForErrorsIn: '2r` a digit between 0 and 1 expected ->`3'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:10'!
testEmptyCaseStatement
	self setUpForErrorsIn: '^ nil caseOf: { ` At least one case required ->`} '.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testExpectedExpressionInBraceArray
	self setUpForErrorsIn: '{ 1. 2 ` Period or right brace expected ->`3 }'.
	self enumerateAllSelections.
	self setUpForErrorsIn: '{ 1. 2. ` Variable or expression expected ->`| x | x}'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testExtraneousStatementAfterAReturnInABlock
	self setUpForErrorsIn: '[ ^1 ` End of block expected ->`2]'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:31'!
testInvalidExternalFunctionDeclaration
	"Not implemented yet.
	#externalFunctionDeclaration skipped, cannot be evaluated"! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:08'!
testInvalidLiteralCharacter
	self setUpForErrorsIn: '^ #yourself , #` Invalid literal character ->`) , #end'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:31'!
testInvalidPattern
	"Not implemented yet.
	#pattern:inContext: skipped, cannot be evaluated"! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:53'!
testInvalidPragma
	"Not implemented yet.
	#pragmaLiteral: #pragmaSequence #pragmaStatement #pragmaPrimitives skipped, cannot be evaluated"! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:31'!
testInvalidPrimitive
	"Not implemented yet.
	##primitive:error: #primitive:module:error: skipped, cannot be evaluated"! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testInvalidRadix
	self setUpForErrorsIn: '1` an integer greater than 1 as valid radix expected ->`r0'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testLiteralCharacterMissing
	self setUpForErrorsIn: '$` A Character was expected ->`'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingArgumentAfterABinaryMessage
	self setUpForErrorsIn: '1 +` Argument expected ->`'.
	self enumerateAllSelections.
	self setUpForErrorsIn: '1 + ` Argument expected ->`* 2 + 3'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingArgumentAfterAMessageKey
	self setUpForErrorsIn: '1 to: ` Argument expected ->`:='.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingBlockArgumentName
	self setUpForErrorsIn: '[ :x : ` Argument name expected ->`1]'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingExpression
	self setUpForErrorsIn: '| x | x := ` Expression expected ->'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingExpressionAfterAReturn
	self setUpForErrorsIn: '^ ` Expression to return expected ->`. 1 + 2'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingMessageAfterACascade
	self setUpForErrorsIn: 'nil yourself; ` Cascade expected ->`^ 2'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 02:06'!
testMissingPeriodSeparatorBetweenStatements
	self setUpForErrorsIn: '1 + 2 ` Nothing more expected ->`^nil'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testMissingSeparatorBetweenBlockArgumentAndStatements
	self setUpForErrorsIn: '[ :x ` Vertical bar expected ->`x + 1 ]'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:40'!
testTempDoubledDefined
	self setUpForErrorsIn: '| x ` Name is already defined ->`x | x := 1. ^x'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testTooLargeAnIntegerInALiteralByteArray
	self setUpForErrorsIn: '#[ 1 2 ` 8-bit integer or right bracket expected ->`256 4 5]'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing-byteCode limits' stamp: 'nice 2/22/2012 01:34'!
testTooManyArguments
	self setUpForErrorsIn: '^[:x1 :x2 :x3 :x4 :x5 :x6 :x7 :x8 :x9 :x10 :x11 :x12 :x13 :x14 :x15 ` Too many arguments ->`:x16 :x17 | ]'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing-byteCode limits' stamp: 'nice 2/22/2012 01:49'!
testTooManyLiterals
	self setUpForErrorsIn: '{#(1). #(2). #(3). #(4). #(5). #(6). #(7). #(8). #(9). #(10). #(11). #(12). #(13). #(14). #(15). #(16). #(17). #(18). #(19). #(20). #(21). #(22). #(23). #(24). #(25). #(26). #(27). #(28). #(29). #(30). #(31). #(32). #(33). #(34). #(35). #(36). #(37). #(38). #(39). #(40). #(41). #(42). #(43). #(44). #(45). #(46). #(47). #(48). #(49). #(50). #(51). #(52). #(53). #(54). #(55). #(56). #(57). #(58). #(59). #(60). #(61). #(62). #(63). #(64). #(65). #(66). #(67). #(68). #(69). #(70). #(71). #(72). #(73). #(74). #(75). #(76). #(77). #(78). #(79). #(80). #(81). #(82). #(83). #(84). #(85). #(86). #(87). #(88). #(89). #(90). #(91). #(92). #(93). #(94). #(95). #(96). #(97). #(98). #(99). #(100). #(101). #(102). #(103). #(104). #(105). #(106). #(107). #(108). #(109). #(110). #(111). #(112). #(113). #(114). #(115). #(116). #(117). #(118). #(119). #(120). #(121). #(122). #(123). #(124). #(125). #(126). #(127). #(128). #(129). #(130). #(131). #(132). #(133). #(134). #(135). #(136). #(137). #(138). #(139). #(140). #(141). #(142). #(143). #(144). #(145). #(146). #(147). #(148). #(149). #(150). #(151). #(152). #(153). #(154). #(155). #(156). #(157). #(158). #(159). #(160). #(161). #(162). #(163). #(164). #(165). #(166). #(167). #(168). #(169). #(170). #(171). #(172). #(173). #(174). #(175). #(176). #(177). #(178). #(179). #(180). #(181). #(182). #(183). #(184). #(185). #(186). #(187). #(188). #(189). #(190). #(191). #(192). #(193). #(194). #(195). #(196). #(197). #(198). #(199). #(200). #(201). #(202). #(203). #(204). #(205). #(206). #(207). #(208). #(209). #(210). #(211). #(212). #(213). #(214). #(215). #(216). #(217). #(218). #(219). #(220). #(221). #(222). #(223). #(224). #(225). #(226). #(227). #(228). #(229). #(230). #(231). #(232). #(233). #(234). #(235). #(236). #(237). #(238). #(239). #(240). #(241). #(242). #(243). #(244). #(245). #(246). #(247). #(248). #(249). #(250). #(251). #(252). #(253). #(254). #(255). #(256). `More than 256 literals referenced. 
You must split or otherwise simplify this method.
The 257th literal is: ->`#(257)}'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing-byteCode limits' stamp: 'nice 2/22/2012 21:52'!
testTooManyTemporaries
	"Nope, with Closure Compiler, an Error is raised before the Encoder has a chance to notify"! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 01:57'!
testUnmatchedBlockBracket
	self setUpForErrorsIn: 'nil yourself. [` Period or right bracket expected ->`'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedBraceArray
	self setUpForErrorsIn: '{ 1. 2` Period or right brace expected ->`'.
	self enumerateAllSelections.
	self setUpForErrorsIn: '{ 1. 2 ` Period or right brace expected ->`'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedByteArrayBracket
	self setUpForErrorsIn: '#` Unmatched bracket ->`[ 1 2 '.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedCommentQuote
	self setUpForErrorsIn: '1+2   ` Unmatched comment quote ->`"unfinished comment'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 02:41'!
testUnmatchedExpressionParenthesis
	self setUpForErrorsIn: '1+(2 ` right parenthesis expected ->`. '.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedLiteralParenthesis
	self setUpForErrorsIn: '#` Unmatched parenthesis ->`( 1 2'.
	self enumerateAllSelections.
	self setUpForErrorsIn: '#` Unmatched parenthesis ->`( 1 2 '.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedLocalTempDeclaration
	self setUpForErrorsIn: '| x y ` Vertical bar expected ->`'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedLocalTempDeclarationInABlock
	self setUpForErrorsIn: '[:z | | x y ` Vertical bar expected ->`]'.
	self enumerateAllSelections.! !

!CompilerNotifyingTest methodsFor: 'testing' stamp: 'nice 2/22/2012 00:55'!
testUnmatchedStringQuote
	self setUpForErrorsIn: '^nil printString ,  ` Unmatched string quote ->`''unfinished string'.
	self enumerateAllSelections! !

!CompilerNotifyingTest methodsFor: 'testing-block arguments' stamp: 'nice 2/22/2012 01:23'!
testifTrueBlockWithArgument
	self setUpForErrorsIn: 'true ifTrue: [:x` <- argument of ifTrue: has too many arguments ->` | 1 + 1 ]'.
	self enumerateAllSelections.
	self setUpForErrorsIn: 'true ifTrue: [:x` <- argument of ifTrue: has too many arguments ->` :y | 1 + 1 ]'.
	self enumerateAllSelections.! !

CompilerNotifyingTest subclass: #CompilerSyntaxErrorNotifyingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CompilerSyntaxErrorNotifyingTest commentStamp: 'nice 2/23/2012 22:09' prior: 0!
A CompilerSyntaxErrorNotifyingTest is a specialization for testing correct handling of non interactive compiler notification.
Non interactive is a very relative notion in Smalltalk...
Here it means that user interaction will not happen directly in the TextEditor holding source code, but rather thru a SyntaxError window that will pop-up.
This test intercept the Notification before the pop-up is raised.
!

!CompilerSyntaxErrorNotifyingTest class methodsFor: 'testing' stamp: 'nice 2/22/2012 22:54'!
shouldInheritSelectors
	"This class can recycle all of super tests, it just has to refine internal Compiler evaluation machinery"
	^true! !

!CompilerSyntaxErrorNotifyingTest methodsFor: 'private' stamp: 'nice 2/23/2012 22:34'!
enumerateAllSelections
	"This method intercepts the SyntaxErrorNotification and prevent the SyntaxError morph to open.
	The notification errorCode hold the source of evaluated sub-selection with inserted error message.
	This can be compared to expected error notification."
	| syntaxErrorProbe |
	syntaxErrorProbe := Object new.
	1 to: self numberOfSelections do: [:n |
		| result |
		result := [self evaluateSelectionNumber: n] on: SyntaxErrorNotification do: [:exc |
			| expectedNotification expectedNotificationLocation |
			expectedNotification := (expectedErrors at: n) allButFirst.
			(expectedNotification endsWith: ' ->') ifTrue: [expectedNotification := expectedNotification allButLast: 3].
			expectedNotificationLocation := (expectedErrorPositions at: n) - (morph editor startIndex - 1).
			self assert: expectedNotificationLocation = exc location.
			self assert: expectedNotification = exc errorMessage asString.
			exc return: syntaxErrorProbe].
		self assert: result == syntaxErrorProbe].! !

!CompilerSyntaxErrorNotifyingTest methodsFor: 'private' stamp: 'nice 2/22/2012 23:08'!
evaluateSelection
	^(nil class evaluatorClass new)
		evaluate: morph editor selection readStream
		"Note subtle difference versus  (morph editor selectionAsStream). 
		The later does not answer the same contents and would raise a SyntaxErrorNotification with wrong sub-selection"
		in: nil
		to: nil
		notifying: nil
		ifFail: [^failure]
		logged: false! !

TestCase subclass: #CompilerTest
	instanceVariableNames: 'errorMessage errorLocation errorSource interactive originalTranscript newTranscript'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CompilerTest commentStamp: 'nice 12/3/2007 22:15' prior: 0!
CompilerTest is a holder for SUnit test of Compiler!

!CompilerTest methodsFor: 'mocking' stamp: 'JorgeRessia 3/4/2010 12:48'!
initializeErrorMessage
	errorMessage := nil.
	errorLocation := nil.
	errorSource := nil! !

!CompilerTest methodsFor: 'mocking' stamp: 'JorgeRessia 3/4/2010 12:49'!
interactive
	^interactive! !

!CompilerTest methodsFor: 'mocking' stamp: 'JorgeRessia 3/4/2010 12:49'!
notify: aString at: aSmallInteger in: aReadStream 
	errorMessage := aString.
	errorLocation := aSmallInteger.
	errorSource := aReadStream.
	! !

!CompilerTest methodsFor: 'running' stamp: 'EstebanLorenzano 8/3/2012 15:28'!
runCase
	SystemAnnouncer uniqueInstance suspendAllWhile: [ super runCase ]
! !

!CompilerTest methodsFor: 'running' stamp: 'simon.denier 6/11/2010 14:24'!
setUp
	originalTranscript := Transcript.
	newTranscript := MockTranscript new.
	Smalltalk globals at: #Transcript put: newTranscript.
	
	! !

!CompilerTest methodsFor: 'running' stamp: 'simon.denier 6/11/2010 14:24'!
tearDown
	Smalltalk globals at: #Transcript put: originalTranscript.
	
	! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'CamilloBruni 1/20/2012 14:32'!
testEmptyCharacterFail
	interactive := true.
	self initializeErrorMessage.

	(Compiler new 
		compile: 'test $'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [^nil]).
	self fail.
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testInBlockArgumentInstanceVariableShadowing
	interactive := true.
	self initializeErrorMessage.

	(Compiler new 
		compile: 'temp [:var1 | ]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 8).
				self assert: (errorSource contents = 'temp [:var1 | ]').
				^nil]).
	self fail.
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testInBlockTempArgumentShadowing
	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp [:temp | |temp|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 16).
				self assert: (errorSource contents = 'temp [:temp | |temp|]').
						^nil]).
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testInBlockTempInstanceVariableShadowing
	interactive := true.
	self initializeErrorMessage.

	(Compiler new 
		compile: 'temp [:temp | |var1|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 16).
				self assert: (errorSource contents = 'temp [:temp | |var1|]').
				^nil]).
	self fail.
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testInBlockTempShadowing

	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp |var2| [:temp| |var2|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 22).
				self assert: (errorSource contents = 'temp |var2| [:temp| |var2|]').
				^nil]).


! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testInstanceVariableShadowing
	interactive := true.
	self initializeErrorMessage.

	Compiler new 
			compile: 'var1 |var1|'
			in: MockForCompilation
			classified: nil
			notifying: self
			ifFail: [
					self assert: (errorMessage = 'Name is already defined ->').
					self assert: (errorLocation = 7).
					self assert: (errorSource contents = 'var1 |var1|').
					^nil].
	self fail.

! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:45'!
testNoShadowing

	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp |var2|'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [self fail.
				^nil]).


! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:44'!
testNotInteractiveInBlockArgumentInstanceVariableShadowing

	self initializeErrorMessage.

	(Compiler new 
		compile: 'temp [:var1 | ]'
		in: MockForCompilation
		classified: nil
		notifying: nil
		ifFail: [self fail.
				^nil]).
		self assert: ( newTranscript contents = '
MockForCompilation>>temp(var1 is shadowed)').
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:44'!
testNotInteractiveInBlockTempArgumentShadowing

	"This test is not completely correct in the sense that it is still interactive. I have to introduce self ( then is interactive) in order to be able to test it"
	
	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp [:temp | |temp|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 16).
				self assert: (errorSource contents = 'temp [:temp | |temp|]').
				^nil]).
	self fail.
! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:44'!
testNotInteractiveInBlockTempInstanceVariableShadowing
	
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp [:temp | |var1|]'
		in: MockForCompilation
		classified: nil
		notifying: nil
		ifFail: [self fail.
				^nil]).

	self assert: ( newTranscript contents = '
MockForCompilation>>temp(var1 is shadowed)').
	! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:44'!
testNotInteractiveInBlockTempShadowing

	"This test is not completely correct in the sense that it is still interactive. I have to introduce self ( then is interactive) in order to be able to test it"
	
	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp |var2| [:temp| |var2|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 22).
				self assert: (errorSource contents = 'temp |var2| [:temp| |var2|]').
				^nil]).


! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'jb 7/1/2011 10:44'!
testNotInteractiveNoShadowing

	self initializeErrorMessage.

	(Compiler new
		compile: 'temp |var2|'
		in: MockForCompilation
		classified: nil
		notifying: nil
		ifFail: [self fail.
				^nil]).
			
	self assert: ( newTranscript contents = '').
	


! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/4/2010 21:46'!
testNotInteractiveShadowingOfTemp

	"This test is not completely correct in the sense that it is still interactive. I have to introduce self ( then is interactive) in order to be able to test it"
	self initializeErrorMessage.

	interactive := false.
	(Compiler new 
		compile: 'temp |temp1 temp1| '
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 13).
				self assert: (errorSource contents = 'temp |temp1 temp1| ').
				^nil]).

	! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/4/2010 21:51'!
testNotInteractiveSiblingBlocksInstanceVariableShadowing

	self initializeErrorMessage.
	
	(Compiler new
		compile: 'temp [:temp | ].[:temp | |var1|]'
		in: MockForCompilation
		classified: nil
		notifying: nil
		ifFail: [self fail.
				^nil]).
			self assert: ( newTranscript contents = '
MockForCompilation>>temp(var1 is shadowed)').


	! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/4/2010 21:51'!
testNotInteractiveSiblingBlocksTempShadowing

	self initializeErrorMessage.

	(Compiler new
		compile: 'temp [:temp | ]. [:temp | ]'
		in: MockForCompilation
		classified: nil
		notifying: nil
		ifFail: [self fail.
						^nil]).
	self assert: ( newTranscript contents = '').



	
					

! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'lr 3/8/2010 15:11'!
testReservedNameAsBlockArgumentShadowing
	interactive := true.
	#( 'self' 'super' 'thisContext' 'true' 'false' 'nil' ) do: [ :each |
		self initializeErrorMessage.
		[ :exit | 
			Compiler new 
				compile: 'temp ^ [ :' , each , ' | ^ ' , each , ' ]'
				in: MockForCompilation
				classified: nil
				notifying: self
				ifFail: [ exit value ].
			self fail ] valueWithExit.
		self assert: errorMessage = 'Name is already defined ->'.
		self assert: errorLocation = 11 ]! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'lr 3/8/2010 15:10'!
testReservedNameAsMethodArgumentShadowing
	interactive := true.
	#( 'self' 'super' 'thisContext' 'true' 'false' 'nil' ) do: [ :each |
		self initializeErrorMessage.
		[ :exit | 
			Compiler new 
				compile: 'temp: ' , each , ' ^ ' , each
				in: MockForCompilation
				classified: nil
				notifying: self
				ifFail: [ exit value ].
			self fail ] valueWithExit.
		self assert: errorMessage = 'Name is already defined ->'.
		self assert: errorLocation = 7 ]! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'TestRunner 3/8/2010 15:10'!
testReservedNameAsTempShadowing
	interactive := true.
	#( 'self' 'super' 'thisContext' 'true' 'false' 'nil' ) do: [ :each |
		self initializeErrorMessage.
		[ :exit | 
			Compiler new 
				compile: 'temp | ' , each , ' | ^ ' , each
				in: MockForCompilation
				classified: nil
				notifying: self
				ifFail: [ exit value ].
			self fail ] valueWithExit.
		self assert: errorMessage = 'Name is already defined ->'.
		self assert: errorLocation = 8 ]! !

!CompilerTest methodsFor: 'literals' stamp: 'nice 12/3/2007 22:20'!
testScaledDecimalLiterals
	"Equal ScaledDecimal with different scales should use different slots
	This is related to http://bugs.squeak.org/view.php?id=6797"
	
	"This correctly works when evaluated separately"
	self deny: (Compiler evaluate: '0.5s1') scale = (Compiler evaluate: '0.5s2') scale.
	
	"But not when evaluated together if literal reduction is too agressive"
	self deny: (Compiler evaluate: '0.5s1 scale =  0.5s2 scale').! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/4/2010 12:55'!
testSiblingBlocksInstanceVariableShadowing

	interactive := true.
	self initializeErrorMessage.
	
	(Compiler new
		compile: 'temp [:temp | ].[:temp | |var1|]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [
				self assert: (errorMessage = 'Name is already defined ->').
				self assert: (errorLocation = 27).
				self assert: (errorSource contents = 'temp [:temp | ].[:temp | |var1|]').
				^nil]).
	self fail.


	! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/4/2010 12:56'!
testSiblingBlocksTempShadowing

	interactive := true.
	self initializeErrorMessage.

	(Compiler new
		compile: 'temp [:temp | ]. [:temp | ]'
		in: MockForCompilation
		classified: nil
		notifying: self
		ifFail: [self fail.
						^nil]).



	
					

! !

!CompilerTest methodsFor: 'test shadowing' stamp: 'JorgeRessia 3/13/2010 16:16'!
testTraitTempShadowing

	self initializeErrorMessage.

	Compiler new 
	compile: 'testReplaceFromToWithStartingAt
	| result  repStart collection replacementCollection firstIndex secondIndex |
		replacementCollection := 1.'
	in: ArrayTest
	classified: nil
	notifying: nil
	ifFail: [self fail.].
	self assert: ( newTranscript contents = '').! !

TestCase subclass: #ContextCompilationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ContextCompilationTest methodsFor: 'tests' stamp: 'eem 6/19/2008 10:11'!
testVariablesAndOffsetsDo

	"ContextCompilationTest new testVariablesAndOffsetsDo"
	| contextClasses |
	contextClasses := ContextPart withAllSuperclasses, ContextPart allSubclasses asArray.
	contextClasses do:
		[:class|
		class variablesAndOffsetsDo:
			[:var :offset|
			self assert: offset < 0.
			self assert: (class instVarNameForIndex: offset negated) == var]].

	InstructionStream withAllSuperclasses, InstructionStream allSubclasses asArray do:
		[:class|
		(contextClasses includes: class) ifFalse:
			[class variablesAndOffsetsDo:
				[:var :offset|
				(InstructionStream instVarNames includes: var) ifFalse:
					[self assert: offset > 0.
					 self assert: (class instVarNameForIndex: offset) == var]]]]! !

TestCase subclass: #CustomParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!CustomParserTest class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 1/10/2011 19:03'!
parserClass

	^ MockCustomParser! !

!CustomParserTest methodsFor: 'tests' stamp: 'StephaneDucasse 1/12/2011 15:42'!
testCustomParser
	"Test whether the compiler uses a custom parser class to parse the source code. This custom parser class is answered by the   class side #parserClass message, instead of default one"
	"self debug: #testCustomParser"
	
	| compiler result |
	compiler := self class compilerClass new.

	"let compiler initialize its class ivar"
	result := compiler parse: ' a ^1 + 1' in: self class notifying: nil.
	
	self assert: compiler parserClass == MockCustomParser
	! !

TestCase subclass: #IfNotNilTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:43'!
testIfNilIfNotNil0Arg

	self assert: (5 ifNil: [#foo] ifNotNil: [#bar]) = #bar.
	self assert: (nil ifNil: [#foo] ifNotNil: [#bar]) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:43'!
testIfNilIfNotNil0ArgAsVar

	| block1 block2 |
	block1 := [#foo].
	block2 := [#bar].
	self assert: (5 ifNil: block1 ifNotNil: block2) = #bar.
	self assert: (nil ifNil: block1 ifNotNil: block2) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/15/2007 12:25'!
testIfNilIfNotNil1Arg

	self assert: (5 ifNil: [#foo] ifNotNil: [:a | a printString]) = '5'.
	self assert: (nil ifNil: [#foo] ifNotNil: [:a | a printString]) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:33'!
testIfNilIfNotNil1ArgAsVar

	| block1 block2 |
	block1 := [#foo].
	block2 := [:a | a printString].
	self assert: (5 ifNil: block1 ifNotNil: block2) = '5'.
	self assert: (nil ifNil: block1 ifNotNil: block2) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
testIfNotNil0Arg
	self assert: (5 ifNotNil: [ #foo ]) = #foo.
	self assert: (nil ifNotNil: [ #foo ]) isNil! !

!IfNotNilTests methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
testIfNotNil0ArgAsVar
	| block |
	block := [ #foo ].
	self assert: (5 ifNotNil: block) = #foo.
	self assert: (nil ifNotNil: block) isNil! !

!IfNotNilTests methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
testIfNotNil1Arg
	self assert: (5 ifNotNil: [ :a | a printString ]) = '5'.
	self assert: (nil ifNotNil: [ :a | a printString ]) isNil! !

!IfNotNilTests methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
testIfNotNil1ArgAsVar
	| block |
	block := [ :a | a printString ].
	self assert: (5 ifNotNil: block) = '5'.
	self assert: (nil ifNotNil: block) isNil! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:35'!
testIfNotNilIfNil0Arg

	self assert: (5 ifNotNil: [#foo] ifNil: [#bar]) = #foo.
	self assert: (nil ifNotNil: [#foo] ifNil: [#bar]) = #bar! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:36'!
testIfNotNilIfNil0ArgAsVar

	| block1 block2 |
	block1 := [#foo].
	block2 := [#bar].
	self assert: (5 ifNotNil: block2 ifNil: block1) = #bar.
	self assert: (nil ifNotNil: block2 ifNil: block1) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:35'!
testIfNotNilIfNil1Arg

	self assert: (5 ifNotNil: [:a | a printString] ifNil: [#foo]) = '5'.
	self assert: (nil ifNotNil: [:a | a printString] ifNil: [#foo]) = #foo! !

!IfNotNilTests methodsFor: 'tests' stamp: 'vb 4/16/2007 11:36'!
testIfNotNilIfNil1ArgAsVar

	| block1 block2 |
	block1 := [#foo].
	block2 := [:a | a printString].
	self assert: (5 ifNotNil: block2 ifNil: block1) = '5'.
	self assert: (nil ifNotNil: block2 ifNil: block1) = #foo! !

TestCase subclass: #MethodNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!MethodNodeTest methodsFor: 'testing' stamp: 'MarcusDenker 4/29/2012 09:51'!
testGenerateWithSource
	| source ast method |
	source := 'testMethod |hello| ^hello. ' .
				ast := Compiler new
					compile: source
					in: self class
					notifying: nil
					ifFail: [self error: 'compilation error'].
	method := ast generateWithSource.				
					
	self assert: (method isKindOf: CompiledMethod).
	self assert: method trailer hasSourcePointer not.
	self assert: method trailer hasSource.
	self assert: (method sourceCode = source).
	
	! !

TestCase subclass: #MirrorPrimitiveTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:47'!
testMirrorAt
	"self debug: #testMirrorAt"
	
	<expectedFailure>
	
	| stackpBefore stackpAfter array byteArray |
	stackpBefore := thisContext stackPtr.
	array := { 1. 2. 3 }.
	byteArray := ByteArray with: 1 with: 2 with: 3.
	self assert: (thisContext object: array basicAt: 1) = 1.
	self assert: (thisContext object: byteArray basicAt: 2) = 2.
	thisContext object: array basicAt: 2 put: #two.
	self assert: array = #(1 #two 3).
	thisContext object: byteArray basicAt: 2 put: 222.
	self assert: byteArray asArray = #(1 222 3).
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter. "Make sure primitives pop all their arguments"
	self should: [thisContext object: array basicAt: 4] raise: Error.
	self should: [thisContext object: byteArray basicAt: 0] raise: Error.
	self should: [thisContext object: byteArray basicAt: 1 put: -1] raise: Error! !

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:48'!
testMirrorClass
	
	<expectedFailure>

	| stackpBefore stackpAfter |
	stackpBefore := thisContext stackPtr.
	self assert: (thisContext objectClass: Array new) = Array.
	self assert: (thisContext objectClass: 1) = 1 class.
	self assert: (thisContext objectClass: ProtoObject new) = ProtoObject.
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter "Make sure primitives pop all their arguments"! !

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:48'!
testMirrorEqEq

	<expectedFailure>

	| stackpBefore stackpAfter |
	stackpBefore := thisContext stackPtr.
	self assert: (thisContext object: Array new eqeq: Array new) == false.
	self assert: (thisContext object: Array eqeq: Array) == true.
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter "Make sure primitives pop all their arguments"! !

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:48'!
testMirrorInstVarAt

	<expectedFailure>

	| stackpBefore stackpAfter array point |
	stackpBefore := thisContext stackPtr.
	array := { 1. 2. 3 }.
	point := Point x: 1 y: 2.
	self assert: (thisContext object: array instVarAt: 1) = 1.
	self assert: (thisContext object: point instVarAt: 2) = 2.
	thisContext object: array instVarAt: 2 put: #two.
	self assert: array = #(1 #two 3).
	thisContext object: point instVarAt: 1 put: 1/2.
	self assert: point = (Point x: 1 / 2 y: 2).
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter. "Make sure primitives pop all their arguments"
	self should: [thisContext object: array instVarAt: 4] raise: Error.
	self should: [thisContext object: point instVarAt: 3] raise: Error! !

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'MarcusDenker 8/20/2011 13:48'!
testMirrorPerform

	<expectedFailure>

	| stackpBefore stackpAfter anInterval |
	stackpBefore := thisContext stackPtr.
	anInterval := 1 to: 2.
	self assert: (thisContext object: anInterval perform:# species withArguments: #() inClass: Interval) == Array.
	self assert: (thisContext object: anInterval perform:# species withArguments: #() inClass: Interval superclass) == Interval.
	self should: [thisContext object: anInterval perform:# species withArguments: #() inClass: Point]
		raise: Error.
	self should: [thisContext object: anInterval perform:# species withArguments: OrderedCollection new inClass: Interval]
		raise: Error.
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter "Make sure primitives pop all their arguments"! !

!MirrorPrimitiveTests methodsFor: 'tests' stamp: 'md 5/1/2012 12:33'!
testMirrorSize

	| stackpBefore stackpAfter |
	stackpBefore := thisContext stackPtr.
	self assert: (thisContext objectSize: #(1 2 3)) = 3.
	self assert: (thisContext objectSize: '123') = 3.
	self assert: (thisContext objectSize: nil) = 0.
	self assert: (thisContext objectSize: 1) = 0.
	stackpAfter := thisContext stackPtr.
	self assert: stackpBefore = stackpAfter. "Make sure primitives pop all their arguments"! !

TestCase subclass: #PerformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!PerformTest commentStamp: 'HenrikSperreJohansen 5/19/2010 02:33' prior: 0!
I test a primitive failure with perform:withArguments: which was raised when selectors with many arguments was called from methods with small stack sizes.!

!PerformTest methodsFor: 'helper methods' stamp: 'Igor.Stasenko 5/19/2010 00:21'!
a1: a1 a2: a2 a3: a3 a4: a4 a5: a5 a6: a6 a7: a7 a8:a8 a9: a9 a10: a10 
a11: a11 a12: a12 a13: a13 a14: a14 a15: a15 

	^ true! !

!PerformTest methodsFor: 'helper methods' stamp: 'MarcusDenker 6/17/2012 15:06'!
doPerformOldLargeFrame
	| t1 t2 t3 t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 |
	"The perform primitive reuses the current stack frame.
	When this was big,the following perform would succeed, since the stack frame was large enough to fit all the arguments"
	
	t1 := t2 := t3 := t4 := t5 := t6 := t7 := t8 := t9 := t10 := t11 := t12 := t13 := 1.
	
	self perform: #a1:a2:a3:a4:a5:a6:a7:a8:a9:a10:a11:a12:a13:a14:a15:
		withArguments: (Array new: 15).

	^ t1! !

!PerformTest methodsFor: 'helper methods' stamp: 'HenrikSperreJohansen 5/19/2010 02:28'!
doPerformOldSmallFrame
	"The perform primitive reuses the current stack frame.
	When this was small, it would cause the perform primitive to fail, when the amount of arguments were too high"
	
	^self perform: #a1:a2:a3:a4:a5:a6:a7:a8:a9:a10:a11:a12:a13:a14:a15:
		withArguments: (Array new: 15).

! !

!PerformTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 5/19/2010 02:39'!
testPerformWithArgsLargeFrame

		self shouldnt: [self doPerformOldLargeFrame] raise: Error.

	 ! !

!PerformTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 5/19/2010 02:39'!
testPerformWithArgsSmallFrame

	self shouldnt: [self doPerformOldSmallFrame] raise: Error.

	 ! !

TestCase subclass: #ScannerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ScannerTest methodsFor: 'testing' stamp: 'jb 7/1/2011 10:54'!
testLiteralSymbols
		
	((32 to: 126) collect: [:ascii | Character value: ascii]) ,
	#(':x:yourself' '::' 'x:yourself' '123' 'x0:1:2:' 'x.y.z' '1abc' 'a1b0c2' ' x' 'x ' '+x-y' '||' '--' '++' '+-' '+/-' '-/+' '<|>' '#x' '()' '[]' '{}' '')
		do: [:str |
			self assert: (self class evaluatorClass evaluate: str asSymbol printString) = str asSymbol
				description: 'in all case, a Symbol must be printed in an interpretable fashion']! !

TestCase subclass: #ScannerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!ScannerTests methodsFor: 'testing' stamp: 'nice 4/4/2010 21:56'!
testAmbiguousSelector
	"Non regression test.
	See http://code.google.com/p/pharo/issues/detail?id=2271
	and http://bugs.squeak.org/view.php?id=7491"
	
	self shouldnt: [Scanner new scanTokens: '1@-1'] raise: Error.
	self assert: ((Scanner new scanTokens: '1@-1') at: 2) asString = '@-'! !

Parser subclass: #MockCustomParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

Object subclass: #AbstractInstructionTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!AbstractInstructionTests commentStamp: 'BenjaminVanRyseghem 9/27/2011 14:04' prior: 0!
Use for a test: ClosureCompilerTest>>#closureCases!

!AbstractInstructionTests methodsFor: 'for test' stamp: 'BenjaminVanRyseghem 9/27/2011 14:07'!
runBinaryConditionalJumps: anObject! !

Object subclass: #MockForCompilation
	instanceVariableNames: 'var1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

Object subclass: #MockTranscript
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!MockTranscript methodsFor: 'accessing' stamp: 'JorgeRessia 3/4/2010 21:19'!
contents
	^stream contents
	! !

!MockTranscript methodsFor: 'streaming' stamp: 'JorgeRessia 3/4/2010 21:19'!
cr 
	stream cr! !

!MockTranscript methodsFor: 'initializing' stamp: 'JorgeRessia 3/4/2010 21:19'!
initialize
 
	super initialize.
	stream := String new writeStream.! !

!MockTranscript methodsFor: 'streaming' stamp: 'JorgeRessia 3/4/2010 21:19'!
show: anObject
	stream nextPutAll: anObject asString
	! !

Object subclass: #TotalCost
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!TotalCost commentStamp: 'StephaneDucasse 6/9/2010 20:53' prior: 0!
Part of an test resource to verify that super is well bound.
See CompilerEvaluationTest!

TotalCost subclass: #AverageCost
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!AverageCost commentStamp: 'StephaneDucasse 6/9/2010 20:53' prior: 0!
Part of an test resource to verify that super is well bound.
See CompilerEvaluationTest!

!AverageCost methodsFor: 'as yet unclassified' stamp: 'NikoSchwarz 6/5/2010 16:11'!
total

	^super total / 12! !

AverageCost subclass: #WeightedAverageCost
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompilerTests'!

!WeightedAverageCost commentStamp: 'StephaneDucasse 6/9/2010 20:53' prior: 0!
Part of an test resource to verify that super is well bound.
See CompilerEvaluationTest!

!TotalCost methodsFor: 'as yet unclassified' stamp: 'NikoSchwarz 6/5/2010 15:53'!
total
	^25000 ! !
